// Copyright 2019-2024 Yuji Tanaka
// HP : http://galaxystar.image.coocan.jp/
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
/*
 * ESP32S3 Web watch for ILI9163
 *
 * 2024'05/21 Yuji Tanaka 
 */
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <ILI9163.h>
#include <WiFi.h>
#include "time.h"

#define TFT_DC 48
#define TFT_CS 47
#define TFT_RST 45

ILI9163 tft = ILI9163(TFT_CS, TFT_DC, TFT_RST);

const char* ssid       = "aterm-c17543-g";
const char* password   = "181411c5886cb";

const char* ntpServer = "pool.ntp.org";
//const long  gmtOffset_sec = 3600;
//const int   daylightOffset_sec = 3600;
const long  gmtOffset_sec = 3600*9;  // JST
const int   daylightOffset_sec = 0;

char week[7][10]={
  "Sunday",
  "Monday",
  "Tuesday",
  "Wednesday",
  "Thursday",
  "Friday",
  "Saturday"
};

char month[12][10]={
 "January",
 "February",
 "March",
 "April",
 "May",
 "June",
 "July",
 "August",
 "September",
 "October",
 "November",
 "December"
};

static int8_t old_sec;

void printLocalTime()
{
  struct tm timeinfo;
  char str[16];

  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  if (timeinfo.tm_sec==old_sec)
    return;
  old_sec=timeinfo.tm_sec;
  
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");

  tft.fillRect(0, 50, 128, 110, ILI9163_BLACK);
  tft.setTextSize(2);

  tft.setTextColor(ILI9163_MAGENTA);
  tft.setCursor(0, 50);
  tft.print(1900+timeinfo.tm_year);
  tft.setTextColor(ILI9163_CYAN);
  tft.setCursor(0, 70);
  tft.print(&month[timeinfo.tm_mon][0]);
  tft.print(" ");
  tft.print(timeinfo.tm_mday);

  tft.setTextColor(ILI9163_YELLOW);
  tft.setCursor(0, 90);
  tft.print(&week[timeinfo.tm_wday][0]);
  
  tft.setTextColor(ILI9163_WHITE);
  tft.setCursor(0, 120);
  sprintf(str, "%02d:%02d:%02d", timeinfo.tm_hour, timeinfo.tm_min, timeinfo.tm_sec);
  tft.print(str);
}

void setup()
{
  Serial.begin(115200);
  
  tft.begin(16000000);
  tft.setRotation(2);
  tft.fillScreen(ILI9163_BLACK);

  //connect to WiFi
  Serial.printf("Connecting to %s ", ssid);

  tft.setTextColor(ILI9163_WHITE);
  tft.setCursor(0,0);
  tft.print("Connecting to ");
  tft.setCursor(0,8);
  tft.print(ssid);
  
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
  }
  Serial.println(" CONNECTED");
  
  tft.setCursor(0,16);
  tft.print("CONNECTED");

  //init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  printLocalTime();

  //disconnect WiFi as it's no longer needed
  WiFi.disconnect(true);
  WiFi.mode(WIFI_OFF);
}

void loop()
{
  delay(100);
  printLocalTime();
}
